@@ -1,6 +1,6 @@
 /* Run an ELF binary on a linux system.
 
-   Copyright (C) 1993-1995, Eric Youngdale.
+   Copyright (C) 1993-1996, Eric Youngdale.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -93,6 +93,7 @@
  */
 
 #include <stdarg.h>
+#include <fcntl.h>
 #ifdef IBCS_COMPATIBLE
 #include <ibcs/unistd.h>
 #else
@@ -108,6 +109,10 @@
 #include "syscall.h"
 #include "string.h"
 
+#include "../config.h"
+
+#define ALLOW_ZERO_PLTGOT
+
 static char * _dl_malloc_addr, *_dl_mmap_zero;
 char * _dl_library_path = 0; /* Where we look for libraries */
 char *_dl_preload = 0; /* Things to be loaded before the libs. */
@@ -169,7 +174,7 @@ typedef struct elf32_rel ELF_RELOC;
  * can set an internal breakpoint on it, so that we are notified when the
  * address mapping is changed in some way.
  */
-static void _dl_debug_state()
+void _dl_debug_state()
 {
   return;
 }
@@ -237,6 +242,14 @@ void _dl_boot(int args){
 #define ZFILENO ((-1 & (~zfileno)) | zfileno)
 /*#define ZFILENO -1*/
 
+#ifndef MAP_ANONYMOUS
+#ifdef __sparc__
+#define MAP_ANONYMOUS 0x20
+#else
+#error MAP_ANONYMOUS not defined and suplementary value not known
+#endif
+#endif
+
     /* See if we need to relocate this address */
     mmap_zero = malloc_buffer = (unsigned char *) _dl_mmap((void*) 0, 4096,
 			     PROT_READ | PROT_WRITE, 
@@ -462,8 +475,10 @@ void _dl_boot(int args){
 	  brk_addr = ppnt->p_vaddr + ppnt->p_memsz;
       }
       if(ppnt->p_type == PT_DYNAMIC) {
+#ifndef ALLOW_ZERO_PLTGOT
 	/* make sure it's really there. */
 	if (app_tpnt->dynamic_info[DT_PLTGOT] == 0) continue;
+#endif
 	/* OK, we have what we need - slip this one into the list. */
 	app_tpnt = _dl_add_elf_hash_table("", 0, 
 			    app_tpnt->dynamic_info, ppnt->p_vaddr, ppnt->p_filesz);
@@ -477,7 +492,10 @@ void _dl_boot(int args){
 	app_tpnt->usage_count++;
 	app_tpnt->symbol_scope = _dl_symbol_tables;
 	lpnt = (int *) (app_tpnt->dynamic_info[DT_PLTGOT]);
-	INIT_GOT(lpnt, _dl_loaded_modules);
+#ifdef ALLOW_ZERO_PLTGOT
+	if (lpnt)
+#endif
+	  INIT_GOT(lpnt, _dl_loaded_modules);
       }
       if(ppnt->p_type == PT_INTERP) { /* OK, fill this in - we did not have
 					 this before */
@@ -498,23 +516,15 @@ void _dl_boot(int args){
 	 (dl_data[AT_UID] != -1 && dl_data[AT_UID] == dl_data[AT_EUID] &&
 	  dl_data[AT_GID] == dl_data[AT_EGID]))
     {
-#ifndef IBCS_COMPATIBLE
-      if (!(_dl_preload = _dl_getenv("LD_ELF_PRELOAD", envp)))
-#endif
-	_dl_preload = _dl_getenv("LD_PRELOAD", envp);
-#ifndef IBCS_COMPATIBLE
-      if (!(_dl_library_path = _dl_getenv("LD_ELF_LIBRARY_PATH", envp)))
-#endif
-	_dl_library_path = _dl_getenv("LD_LIBRARY_PATH",envp);
+      _dl_preload = _dl_getenv("LD_PRELOAD", envp);
+      _dl_library_path = _dl_getenv("LD_LIBRARY_PATH",envp);
     }
     else
     {
       _dl_unsetenv("LD_PRELOAD", envp);
-      _dl_unsetenv("LD_ELF_PRELOAD", envp);
       _dl_unsetenv("LD_AOUT_PRELOAD", envp);
       _dl_preload = NULL;
       _dl_unsetenv("LD_LIBRARY_PATH", envp);
-      _dl_unsetenv("LD_ELF_LIBRARY_PATH", envp);
       _dl_unsetenv("LD_AOUT_LIBRARY_PATH", envp);
       _dl_library_path = NULL;
     }
@@ -533,8 +543,8 @@ void _dl_boot(int args){
 
   {
     struct elf_resolve *tcurr;
-    struct elf_resolve * tpnt1;
-    char * lpnt;
+    struct elf_resolve *tpnt1;
+    char *lpnt;
 
     if (_dl_preload)  {
       int i;
@@ -576,6 +586,85 @@ void _dl_boot(int args){
       }
     }
 
+#ifndef IBCS_COMPATIBLE
+    {
+      int fd;
+      struct stat st;
+      char *preload;
+
+      if (!_dl_stat(LDSO_PRELOAD, &st)) {
+	if ((fd = _dl_open(LDSO_PRELOAD, O_RDONLY)) < 0) {
+	  _dl_fdprintf(2, "%s: can't open file '%s'\n", _dl_progname, 
+		       LDSO_PRELOAD);
+	} else {
+	  preload = (caddr_t)_dl_mmap(0, st.st_size+1, PROT_READ|PROT_WRITE, 
+				      MAP_PRIVATE, fd, 0);
+	  _dl_close (fd);
+	  if (preload == (caddr_t)-1) {
+	    _dl_fdprintf(2, "%s: can't map file '%s'\n", _dl_progname, 
+			 LDSO_PRELOAD);
+	  } else {
+	    char c, *cp, *cp2;
+
+	    /* convert all separators and comments to spaces */
+	    for (cp = preload; *cp; /*nada*/) {
+	      if (*cp == ':' || *cp == '\t' || *cp == '\n') {
+		*cp++ = ' ';
+	      } else if (*cp == '#') {
+		do
+		  *cp++ = ' ';
+		while (*cp != '\n' && *cp != '\0');
+	      } else {
+		cp++;
+	      }
+	    }
+
+	    /* find start of first library */
+	    for (cp = preload; *cp && *cp == ' '; cp++)
+	      /*nada*/;
+
+	    while (*cp) {
+	      /* find end of library */
+	      for (cp2 = cp; *cp && *cp != ' '; cp++)
+		/*nada*/;
+	      c = *cp;
+	      *cp = '\0';
+
+	      tpnt1 = _dl_load_shared_library(NULL, cp2);
+	      if (!tpnt1) {
+		if (_dl_trace_loaded_objects)
+		  _dl_fdprintf(1, "\t%s => not found\n", cp2);
+		else {
+		  _dl_fdprintf(2, "%s: can't load library '%s'\n",
+			       _dl_progname, cp2);
+		  _dl_exit(15);
+		}
+	      } else {
+		if (_dl_trace_loaded_objects && !tpnt1->usage_count)
+		  _dl_fdprintf(1, "\t%s => %s\n", cp2, tpnt1->libname);
+                rpnt->next = 
+		  (struct dyn_elf *) _dl_malloc(sizeof(struct dyn_elf));
+		_dl_memset (rpnt->next, 0, sizeof (*(rpnt->next)));
+		rpnt = rpnt->next;
+		tpnt1->usage_count++;
+		tpnt1->symbol_scope = _dl_symbol_tables;
+		tpnt1->libtype = elf_lib;
+		rpnt->dyn = tpnt1;
+	      }
+
+	      /* find start of next library */
+	      *cp = c;
+	      for (/*nada*/; *cp && *cp == ' '; cp++)
+		/*nada*/;
+	    }
+
+	    _dl_munmap(preload, st.st_size+1);
+	  }
+	}
+      }
+    }
+#endif
+
     for (tcurr = _dl_loaded_modules; tcurr; tcurr = tcurr->next)
     {
       for (dpnt = (struct dynamic *)tcurr->dynamic_addr; dpnt->d_tag; dpnt++)
@@ -745,6 +834,10 @@ void _dl_boot(int args){
   debug_addr->r_ldbase = load_addr;
   debug_addr->r_brk = (unsigned long) &_dl_debug_state;
   _dl_debug_addr = debug_addr;
+  debug_addr->r_state = RT_CONSISTENT;
+  /* This is written in this funny way to keep gcc from inlining the
+     function call. */
+  ((void (*)(void))debug_addr->r_brk)();
 
   for(tpnt = _dl_loaded_modules; tpnt; tpnt = tpnt->next)
     {
