@@ -1,6 +1,6 @@
 /* Run an ELF binary on a linux system.
 
-   Copyright (C) 1993, Eric Youngdale.
+   Copyright (C) 1993-1996, Eric Youngdale.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -81,11 +81,14 @@ unsigned long _dl_elf_hash(const char * name){
  */
 struct elf_resolve * _dl_check_hashed_files(char * libname){
   struct elf_resolve * tpnt;
-  tpnt = _dl_loaded_modules;
-  while(tpnt){
-    if(_dl_strcmp(tpnt->libname, libname) == 0) return tpnt;
-    tpnt = tpnt->next;
-  };
+  int len = _dl_strlen(libname);
+
+  for (tpnt = _dl_loaded_modules; tpnt; tpnt = tpnt->next) {
+    if (_dl_strncmp(tpnt->libname, libname, len) == 0 &&
+	(tpnt->libname[len] == '\0' || tpnt->libname[len] == '.'))
+      return tpnt;
+  }
+
   return NULL;
 }
 
@@ -158,17 +161,10 @@ char * _dl_find_hash(char * name, struct dyn_elf * rpnt1,
   char * weak_result;
   struct elf_resolve * first_def;
   struct dyn_elf * rpnt, first;
-  int next;
 
   weak_result = 0;
   elf_hash_number = _dl_elf_hash(name);
 
-  if(rpnt1 == (struct dyn_elf*)RTLD_NEXT) {
-      rpnt1 = 0;
-      next = 1;
-  }
-  else next = 0;
-
   /* A quick little hack to make sure that any symbol in the executable
   will be preferred to one in a shared library.  This is necessary so
   that any shared library data symbols referenced in the executable
@@ -267,10 +263,6 @@ char * _dl_find_hash(char * name, struct dyn_elf * rpnt1,
 	      if(first_def == f_tpnt && symtab[si].st_shndx == 0)
 		continue;
 	    }
-	    if(next) {
-	      next = 0;
-	      continue;
-	    }
 	    
 	    switch(ELF32_ST_BIND(symtab[si].st_info)){
 	    case STB_GLOBAL:
