@@ -1,6 +1,6 @@
 /* Load an ELF sharable library into memory.
 
-   Copyright (C) 1993, Eric Youngdale.
+   Copyright (C) 1993-1996, Eric Youngdale.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -53,8 +53,10 @@ int _dl_map_cache(void)
   struct stat st;
   header_t *header;
 
-  if (_dl_cache_addr != NULL)
+  if (_dl_cache_addr == (caddr_t)-1)
     return -1;
+  else if (_dl_cache_addr != NULL)
+    return 0;
 
   if (_dl_stat(LDSO_CACHE, &st) || (fd = _dl_open(LDSO_CACHE, O_RDONLY)) < 0)
   {
@@ -101,8 +103,10 @@ int _dl_unmap_cache(void)
   if (_dl_cache_addr == NULL || _dl_cache_addr == (caddr_t)-1)
     return -1;
 
+#if 1
   _dl_munmap (_dl_cache_addr, _dl_cache_size);
   _dl_cache_addr = NULL;
+#endif
 
   return 0;
 }
@@ -119,7 +123,7 @@ unsigned int _dl_internal_error_number;
 struct elf_resolve * _dl_load_shared_library(struct elf_resolve * tpnt,
 	char * full_libname){
   char * pnt, *pnt1, *pnt2;
-  struct elf_resolve *tpnt1;
+  struct elf_resolve *tpnt1 = NULL;
   char mylibname[1024];
   char * libname;
 
@@ -160,9 +164,9 @@ struct elf_resolve * _dl_load_shared_library(struct elf_resolve * tpnt,
    * the default path of /usr/lib.
    * Check in rpath directories 
    */
-  if( tpnt != NULL )
-    {
-      pnt1 = (char *) tpnt->dynamic_info[DT_RPATH];
+  for (tpnt = _dl_loaded_modules; tpnt; tpnt = tpnt->next) {
+    if (tpnt->libtype == elf_executable) {
+      pnt1 = (char *)tpnt->dynamic_info[DT_RPATH];
       if(pnt1) {
 	pnt1 += (unsigned int) tpnt->loadaddr + tpnt->dynamic_info[DT_STRTAB];
 	while(*pnt1){
@@ -178,6 +182,7 @@ struct elf_resolve * _dl_load_shared_library(struct elf_resolve * tpnt,
 	}
       }
     }
+  }
   
 
   /* Check in LD_{ELF_}LIBRARY_PATH, if specified and allowed */
@@ -260,6 +265,8 @@ goof:
  * are required.
  */
 
+extern _elf_rtbndr(void);
+
 struct elf_resolve * _dl_load_elf_shared_library(char * libname, int flag){
   struct elfhdr * epnt;
   unsigned int dynamic_addr = 0;
@@ -371,7 +378,7 @@ struct elf_resolve * _dl_load_elf_shared_library(char * libname, int flag){
   };
   libaddr=(unsigned int)status;
   flags|=MAP_FIXED;
-  
+
   /* Get the memory to store the library */
   ppnt = (struct elf_phdr *) &header[epnt->e_phoff];
   
@@ -390,7 +397,7 @@ struct elf_resolve * _dl_load_elf_shared_library(char * libname, int flag){
 	unsigned int map_size;
 	char * cpnt;
 	
-	status = (char *) _dl_mmap((char *) (libaddr  +
+	status = (char *) _dl_mmap((char *) ((piclib?libaddr:0)  +
 					     (ppnt->p_vaddr & 0xfffff000)),
 			  (ppnt->p_vaddr & 0xfff) + ppnt->p_filesz, 
 			  LXFLAGS(ppnt->p_flags), 
@@ -405,25 +412,23 @@ struct elf_resolve * _dl_load_elf_shared_library(char * libname, int flag){
 	    return NULL;
 	};
 	
-	if(!piclib) status = 0;
-	
 	/* Pad the last page with zeroes. */
 	cpnt =(char *) (status + (ppnt->p_vaddr & 0xfff) + ppnt->p_filesz);
 	while(((unsigned int) cpnt) & 0xfff) *cpnt++ = 0;
-	
+
 /* I am not quite sure if this is completely correct to do or not, but
    the basic way that we handle bss segments is that we mmap /dev/zero if
    there are any pages left over that are not mapped as part of the file */
 
 	map_size = (ppnt->p_vaddr + ppnt->p_filesz + 0xfff) & 0xfffff000;
 	if(map_size < ppnt->p_vaddr + ppnt->p_memsz)
-	  status = (char *) _dl_mmap((char *) map_size + libaddr, 
+	  status = (char *) _dl_mmap((char *) map_size + (piclib?libaddr:0), 
 			    ppnt->p_vaddr + ppnt->p_memsz - map_size,
 			    LXFLAGS(ppnt->p_flags),
 			    flags | MAP_ANONYMOUS, -1, 0);
       } else
 	status = (char *) _dl_mmap((char *) (ppnt->p_vaddr & 0xfffff000) + 
-				   libaddr, 
+				   (piclib?libaddr:0), 
 			  (ppnt->p_vaddr & 0xfff) + ppnt->p_filesz, 
 			  LXFLAGS(ppnt->p_flags), 
 			  flags, infile, 
@@ -435,6 +440,7 @@ struct elf_resolve * _dl_load_elf_shared_library(char * libname, int flag){
 	_dl_close(infile);
 	return NULL;
       };
+
       /* if(libaddr == 0 && piclib) {
 	libaddr = (unsigned int) status;
 	flags |= MAP_FIXED;
@@ -445,9 +451,9 @@ struct elf_resolve * _dl_load_elf_shared_library(char * libname, int flag){
   _dl_close(infile);
   
   /* For a non-PIC library, the addresses are all absolute */
-  if(!piclib) libaddr = 0;
-
-  dynamic_addr += (unsigned int) libaddr;
+  if(piclib) {
+    dynamic_addr += (unsigned int) libaddr;
+  }
 
  /* 
   * OK, the ELF library is now loaded into VM in the correct locations
@@ -482,7 +488,7 @@ struct elf_resolve * _dl_load_elf_shared_library(char * libname, int flag){
   ppnt = (struct elf_phdr *) &header[epnt->e_phoff];
   for(i=0;i < epnt->e_phnum; i++, ppnt++){
     if(ppnt->p_type == PT_LOAD && !(ppnt->p_flags & PF_W))
-      _dl_mprotect((void *) (libaddr + (ppnt->p_vaddr & 0xfffff000)),
+      _dl_mprotect((void *) ((piclib?libaddr:0) + (ppnt->p_vaddr & 0xfffff000)),
 		   (ppnt->p_vaddr & 0xfff) + (unsigned int) ppnt->p_filesz,
 		   PROT_READ | PROT_WRITE | PROT_EXEC);
   }
