@@ -4,6 +4,6 @@ main () {
 char hn[10];
 
 	gethostname(hn,10);
-	printf("got hostid=%ld, hosname=%s\n",gethostid(), hn);
+	printf("got hostid=%ld, hostname=%s\n",gethostid(), hn);
 
 }
