@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1994, 1995 Eric Youngdale, Peter MacDonald, David Engel,
+ * Copyright (c) 1994-1996 Eric Youngdale, Peter MacDonald, David Engel,
  * Hongjiu Lu and Mitch D'Souza
  *
  * All rights reserved.
@@ -48,8 +48,6 @@
  * conjunction with the program executable.
  */
 
-#define exit libc_exit
-
 #include <alloca.h>
 #include <unistd.h>
 #include <syscall.h>
@@ -59,14 +57,12 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <limits.h>
-#include <linux/string.h>
+#include <string.h>
 #include <sys/stat.h>
 #include "../config.h"
 #include "fixups.h"
 #include <sys/mman.h>
 
-#undef exit
-
 #define EXIT_FATAL 128
 
 #define VMAJOR(x) (((x) & MAJOR_MASK) >> 16)
@@ -79,13 +75,15 @@
 
 int errno = 0;
 char **__environ = 0;
-char *___strtok=NULL;
-extern int _entry;
+#pragma weak environ = __environ
+/*char *___strtok=NULL;*/
+extern int _start;
 extern int _edata;
 
 int fdprintf(int fd, const char *fmt, ...);
 
-_syscall1(int,exit,int,code)
+#define SYS_dl_exit SYS_exit
+_syscall1(int,dl_exit,int,code)
 
 static caddr_t cache_init (char *, size_t *);
 
@@ -145,8 +143,8 @@ save_mapinfo(unsigned *mapinfo)
 {
     if (mapinfo)
     {
-	mapinfo[0] = (unsigned)&_entry-0x20;
-	mapinfo[1] = ((unsigned)&_edata-(unsigned)&_entry+0xfff) & ~0xfff;
+	mapinfo[0] = (unsigned)&_start-0x20;
+	mapinfo[1] = ((unsigned)&_edata-(unsigned)&_start+0xfff) & ~0xfff;
     }
 }
 #endif
@@ -160,9 +158,9 @@ shared_loader(int func, ...)
     char *cp;
     int i, ldd = 0;
     va_list ap;
-    struct libentry **__SHARED_LIBRARIES__;
-    struct fixuplist *_SHARABLE_CONFLICTS__;
-    char *argv0;
+    struct libentry **__SHARED_LIBRARIES__ = NULL;
+    struct fixuplist *_SHARABLE_CONFLICTS__ = NULL;
+    char *argv0 = NULL;
 #ifndef DEBUG_LIB
     unsigned *mapinfo = NULL;
 #endif
@@ -218,7 +216,7 @@ shared_loader(int func, ...)
 	/* you want me to do what? */
 	fdprintf(2, "%s: invalid dynamic linker option %d\n",
 		 LDSO_IMAGE, func);
-	exit(EXIT_FATAL);
+	dl_exit(EXIT_FATAL);
     }
 
     /* read any environmental options */
@@ -233,10 +231,8 @@ shared_loader(int func, ...)
     if (getuid() == geteuid() && getgid() == getegid())
     {
       /* Well OK, I'll allow it, but just this time */
-      if (!(preload = getenv("LD_AOUT_PRELOAD")))
-        preload = getenv("LD_PRELOAD");
-      if (!(cp = getenv("LD_AOUT_LIBRARY_PATH")))
-        cp = getenv("LD_LIBRARY_PATH");
+      preload = getenv("LD_AOUT_PRELOAD");
+      cp = getenv("LD_AOUT_LIBRARY_PATH");
       if (cp && *cp)
       {
 	char *temp = alloca(strlen(cp) + 1);
@@ -247,11 +243,9 @@ shared_loader(int func, ...)
     {
       /* sorry, Charlie, I can't let you do that */
       unsetenv("LD_PRELOAD");
-      unsetenv("LD_ELF_PRELOAD");
       unsetenv("LD_AOUT_PRELOAD");
       preload = NULL;
       unsetenv("LD_LIBRARY_PATH");
-      unsetenv("LD_ELF_LIBRARY_PATH");
       unsetenv("LD_AOUT_LIBRARY_PATH");
     }
 
@@ -319,7 +313,7 @@ shared_loader(int func, ...)
 	    else
 	    {
 		fdprintf(2, "%s: can't find library '%s'\n", argv0, cp);
-		exit(EXIT_FATAL);
+		dl_exit(EXIT_FATAL);
 	    }
 	}
 	else
@@ -361,7 +355,7 @@ shared_loader(int func, ...)
 			     buffer);
 		    fdprintf(2, "\tRequire major version %d and found %d\n",
 			     VMAJOR((*ptr)->vers), VMAJOR(*vers));
-		    exit(EXIT_FATAL);
+		    dl_exit(EXIT_FATAL);
 		    break;
 		}
 	    }
@@ -385,27 +379,27 @@ shared_loader(int func, ...)
 	if (__do_fixups(_SHARABLE_CONFLICTS__, 0))
 	{
 	    fdprintf(2, "%s: %s fixup pass 1\n", argv0, err);
-	    exit(EXIT_FATAL);
+	    dl_exit(EXIT_FATAL);
 	}
 
 	if (__dynamic_resolve (_SHARABLE_CONFLICTS__))
 	{
 	    fdprintf(2, "%s: %s resolve pass\n", argv0, err);
-	    exit(EXIT_FATAL);
+	    dl_exit(EXIT_FATAL);
 	}
 
 	/* Add (possibly) new addr to .long _foo */
 	if (__do_fixups(_SHARABLE_CONFLICTS__, 1))
 	{
 	    fdprintf(2, "%s: %s fixup pass 2\n", argv0, err);
-	    exit(EXIT_FATAL);
+	    dl_exit(EXIT_FATAL);
 	}
 
 	/* Set the magic numbers back to the starting values for emacs */
 	if (__reset_magic(_SHARABLE_CONFLICTS__))
 	{
 	    fdprintf(2, "%s: %s reset pass\n", argv0, err);
-	    exit(EXIT_FATAL);
+	    dl_exit(EXIT_FATAL);
 	}
     }
     if (preload)
