@@ -1,7 +1,7 @@
 /*
  * ldconfig - update shared library symlinks
  *
- * usage: ldconfig [-DvnNX] dir ...
+ * usage: ldconfig [-DvnNX] [-f conf] [-C cache] dir ...
  *        ldconfig -l [-Dv] lib ...
  *        ldconfig -p
  *        -D: debug mode, don't update links
@@ -11,10 +11,12 @@
  *        -X: don't update the library links
  *        -l: library mode, manually link libraries
  *        -p: print the current library cache
+ *        -f conf: use conf instead of /etc/ld.so.conf
+ *        -C cache: us cache instead of /etc/ld.so.cache
  *        dir ...: directories to process
  *        lib ...: libraries to link
  *
- * Copyright 1994, 1995 David Engel and Mitch D'Souza
+ * Copyright 1994-1996 David Engel and Mitch D'Souza
  *
  * This program may be used for any purpose as long as this
  * copyright notice is kept.
@@ -60,6 +62,7 @@ int libmode = 0;		/* library mode */
 int nocache = 0;		/* don't build cache */
 int nolinks = 0;		/* don't update links */
 
+char *conffile = LDSO_CONF;	/* default cache file */
 char *cachefile = LDSO_CACHE;	/* default cache file */
 void cache_print(void);
 void cache_dolib(char *dir, char *so, char *lib, int);
@@ -128,7 +131,7 @@ char *is_shlib(char *dir, char *name, int *type)
     char buff[1024];
 
     /* see if name is of the form libZ.so* */
-    if (strncmp(name, "lib", 3) == 0 &&
+    if (strncmp(name, "lib", 3) == 0 && name[strlen(name)-1] != '~' &&
 	(cp = strstr(name, ".so")))
     {
 	/* find the start of the Vminor part, if any */
@@ -144,22 +147,13 @@ char *is_shlib(char *dir, char *name, int *type)
 	/* first, make sure it's a regular file */
 	if (lstat(buff, &statbuf))
 	    warn("can't lstat %s (%s), skipping", buff, strerror(errno));
-	else if (S_ISLNK(statbuf.st_mode) && stat(buff, &statbuf))
-	{
-	    /* we've got a stale link.  if there is no version number, 
-	       assume it's for ld and leave it alone, otherwise try
-	       to remove it */
-	    if ((cp2 = strstr(name, ".so")) && cp2[3])
-	    {
-	        if (verbose)
-		    warn("%s is a stale symlink, removing", buff);
-		if (unlink(buff))
-		   warn("can't unlink %s (%s), skipping", buff,
-			strerror(errno));
-	    }
-	}
 	else if (!S_ISREG(statbuf.st_mode))
-	    warn("%s is not a regular file, skipping", buff, strerror(errno));
+	{
+	    /* silently ignore symlinks */
+	    if (!S_ISLNK(statbuf.st_mode))
+	        warn("%s is not a regular file, skipping", 
+		     buff, strerror(errno));
+	}
 	else
 	{
 	    /* then try opening it */
@@ -189,6 +183,8 @@ char *is_shlib(char *dir, char *name, int *type)
 			    if (strncmp(good, name, len) != 0 ||
 				(name[len] != '\0' && name[len] != '.'))
 			    {
+			        warn("%s has inconsistent soname (%s), "
+				     "skipping", buff, good);
 			        free(good);
 				good = NULL;
 			    }
@@ -371,20 +367,31 @@ void scan_dir(char *name)
 /* return the list of system-specific directories */
 char *get_extpath(void)
 {
-    char *cp = NULL;
+    char *res = NULL, *cp;
     FILE *file;
     struct stat stat;
 
-    if ((file = fopen(LDSO_CONF, "r")) != NULL)
+    if ((file = fopen(conffile, "r")) != NULL)
     {
 	fstat(fileno(file), &stat);
-	cp = xmalloc(stat.st_size + 1);
-	fread(cp, 1, stat.st_size, file);
+	res = xmalloc(stat.st_size + 1);
+	fread(res, 1, stat.st_size, file);
 	fclose(file);
-	cp[stat.st_size] = '\0';
+	res[stat.st_size] = '\0';
+
+	/* convert comments fo spaces */
+	for (cp = res; *cp; /*nada*/) {
+	  if (*cp == '#') {
+	    do
+	      *cp++ = ' ';
+	    while (*cp && *cp != '\n');
+	  } else {
+	    cp++;
+	  }
+	}	  
     }
 
-    return cp;
+    return res;
 }
 
 int main(int argc, char **argv)
@@ -399,7 +406,7 @@ int main(int argc, char **argv)
     prog = argv[0];
     opterr = 0;
 
-    while ((c = getopt(argc, argv, "DvnNXlpf:")) != EOF)
+    while ((c = getopt(argc, argv, "DvnNXlpf:C:")) != EOF)
 	switch (c)
 	{
 	case 'D':
@@ -427,8 +434,14 @@ int main(int argc, char **argv)
 	case 'p':
 	    printcache = 1;	/* print cache */
 	    break;
+	case 'f':
+	    conffile = optarg;	/* alternate conf file */
+	    break;
+	case 'C':
+	    cachefile = optarg;	/* alternate cache file */
+	    break;
 	default:
-	    fprintf(stderr, "usage: %s [-DvnNX] dir ...\n", prog);
+	    fprintf(stderr, "usage: %s [-DvnNX] [-f conf] [-C cache] dir ...\n", prog);
 	    fprintf(stderr, "       %s -l [-Dv] lib ...\n", prog);
 	    fprintf(stderr, "       %s -p\n", prog);
 	    exit(EXIT_FATAL);
@@ -436,9 +449,6 @@ int main(int argc, char **argv)
 
 	/* THE REST OF THESE ARE UNDOCUMENTED AND MAY BE REMOVED
 	   IN FUTURE VERSIONS. */
-	case 'f':
-	    cachefile = optarg;	/* alternate cache file */
-	    break;
 	}
 
     /* allow me to introduce myself, hi, my name is ... */
