@@ -9,7 +9,7 @@
  *	      report any missing objects (ELF only).
  *        prog ...: programs to check
  *
- * Copyright 1993, David Engel
+ * Copyright 1993-1996, David Engel
  *
  * This program may be used for any purpose as long as this
  * copyright notice is kept.
@@ -57,19 +57,28 @@ int is_bin(char *argv0, char *prog)
 		struct elf_phdr phdr;
 		int i;
 
-		/* Check its an ELF exectuable and not an object. We assume it
-		   is static currently until we locate a .dynamic section */
-		res = elf_hdr.e_type == ET_EXEC ? 3 : 0;
-
-		/* determine if it is dynamic ELF */
-		for (i=0; i<elf_hdr.e_phnum; i++)
+		/* Check its an exectuable, library or other */
+		switch (elf_hdr.e_type)
 		{
-		    fread(&phdr, sizeof phdr, 1, file);
-		    if (phdr.p_type == PT_DYNAMIC)
+		  case ET_EXEC:
+		    res = 3;
+		    /* then determine if it is dynamic ELF */
+		    for (i=0; i<elf_hdr.e_phnum; i++)
 		    {
-			res = 2;
-			break;
+		        fread(&phdr, sizeof phdr, 1, file);
+			if (phdr.p_type == PT_DYNAMIC)
+			{
+			    res = 2;
+			    break;
+			}
 		    }
+		    break;
+		  case ET_DYN:
+		    res = 4;
+		    break;
+		  default:
+		    res = 0;
+		    break;
 		}
 	    }
 	}
@@ -88,6 +97,8 @@ int main(int argc, char **argv, char **envp)
     int resolve = 0;
     int bind = 0;
     int bintype;
+    char *ld_preload;
+    int status = 0;
 
     /* this must be volatile to work with -O, GCC bug? */
     volatile loadptr loader = (loadptr)LDSO_ADDR;
@@ -126,6 +137,14 @@ int main(int argc, char **argv, char **envp)
 	exit(0);
     }
 
+    /* setup the environment for ELF binaries */
+    putenv("LD_TRACE_LOADED_OBJECTS=1");
+    if (resolve || bind)
+        putenv("LD_BIND_NOW=1");
+    if (resolve)
+        putenv("LD_WARN=1");
+    ld_preload = getenv("LD_PRELOAD");
+
     /* print the dependencies for each program */
     for (i = optind; i < argc; i++)
     {
@@ -134,7 +153,10 @@ int main(int argc, char **argv, char **envp)
 
 	/* make sure it's a binary file */
 	if (!(bintype = is_bin(argv[0], argv[i])))
+	{
+	    status = 1;
 	    continue;
+	}
 
 	/* print the program name if doing more than one */
 	if (optind < argc-1)
@@ -143,26 +165,13 @@ int main(int argc, char **argv, char **envp)
 	    fflush(stdout);
 	}
 
+	/* no need to fork/exec for static ELF program */
 	if (bintype == 3)
 	{
 	    printf("\tstatically linked (ELF)\n");
 	    continue;
 	}
 
-	if (bintype == 2)
-	{	    
-	    /* For ELF binaries we do this instead */
-	    putenv("LD_TRACE_LOADED_OBJECTS=1");
-	    if (resolve || bind)
-	        putenv("LD_BIND_NOW=1");
-	    if (resolve)
-	        putenv("LD_WARN=1");
-	}
-
-	/* save the name in the enviroment, ld.so may need it */
-	sprintf(buff, "%s=%s", LDD_ARGV0, argv[i]);
-	putenv(buff);
-
 	/* now fork and exec prog with argc = 0 */
 	if ((pid = fork()) < 0)
 	{
@@ -171,10 +180,33 @@ int main(int argc, char **argv, char **envp)
 	}
 	else if (pid == 0)
 	{
-	    if (bintype == 2)
-		execl(argv[i], argv[i], NULL);
-	    else
+	    switch (bintype)
+	    {
+	      case 1: /* a.out */
+	        /* save the name in the enviroment, ld.so may need it */
+	        snprintf(buff, sizeof buff, "%s=%s", LDD_ARGV0, argv[i]);
+		putenv(buff);
 		execl(argv[i], NULL);
+		break;
+	      case 2: /* ELF program */
+		execl(argv[i], argv[i], NULL);
+		break;
+	      case 4: /* ELF library */
+	        /* add library to LD_PRELOAD and execute lddstub */
+		if (ld_preload && *ld_preload)
+		    snprintf(buff, sizeof buff, "%s=%s:%s%s", "LD_PRELOAD",
+			     ld_preload, *argv[i] == '/' ? "" : "./", argv[i]);
+		else
+		    snprintf(buff, sizeof buff, "%s=%s%s", "LD_PRELOAD", 
+			     *argv[i] == '/' ? "" : "./", argv[i]);
+		putenv(buff);
+		execl(LDDSTUB, argv[i], NULL);
+		break;
+	      default:
+		fprintf(stderr, "%s: internal error, bintype = %d\n",
+			argv[0], bintype);
+		exit(1);
+	    }
 	    fprintf(stderr, "%s: can't execute %s (%s)\n", argv[0], argv[i],
 		    strerror(errno));
 	    exit(1);
@@ -196,5 +228,5 @@ int main(int argc, char **argv, char **envp)
 	}
     }
 
-    exit(0);
+    exit(status);
 }
